#include <stdio.h>
#include <string.h>
#include <openssl/evp.h>
#include <openssl/engine.h>

int main()
{
    ENGINE *e = NULL;
    EVP_MD_CTX *mdctx;
    const EVP_MD *md;
    int retval;
    char *msg = "KylinSoft\n";
    unsigned char *digest;
    int md_len;

    ENGINE_load_builtin_engines();
    e = ENGINE_by_id("scto");
    if (NULL == e) {
    	fprintf(stderr, "Failed to load ENGINE");
	return -1;
    }

    mdctx = EVP_MD_CTX_new();
    if (NULL == mdctx) {
        fprintf(stderr, "Failed to EVP_MD_CTX_new");
        return -1;
    }

    md = EVP_sm3();
    if (NULL == md) {
        fprintf(stderr, "Failed to EVP_sm3");
        return -1;
    }

    retval = EVP_DigestInit_ex(mdctx, md, e);
    if (1 != retval) {
        fprintf(stderr, "Failed to EVP_DigestInit_ex");
        return -1;
    }

    retval = EVP_DigestUpdate(mdctx, msg, strlen(msg));
    if (1 != retval) {
        fprintf(stderr, "Failed to EVP_DigestUpdate");
        return -1;
    }

    digest = OPENSSL_malloc(EVP_MD_size(md));
    if (NULL == digest) {
        fprintf(stderr, "Failed to OPENSSL_malloc");
        return -1;
    }

    retval = EVP_DigestFinal_ex(mdctx, digest, &md_len);
    if (1 != retval) {
        fprintf(stderr, "Failed to EVP_DigestFinal_ex");
        return -1;
    }

    printf("Digest is: ");
    for (int i = 0; i < md_len; i++)
        printf("%02x", digest[i]);
    printf("\n");

    OPENSSL_free(digest);
    ENGINE_free(e);

    EVP_MD_CTX_free(mdctx);
}
